@@ -80,9 +80,9 @@
     clippy::verbose_file_reads,
     clippy::zero_sized_map_values,
     future_incompatible,
-    missing_crate_level_docs,
     nonstandard_style,
-    rust_2018_idioms
+    rust_2018_idioms,
+    rustdoc::missing_crate_level_docs
 )]
 #![allow(clippy::float_cmp)]
 #![allow(clippy::manual_range_contains)]
@@ -97,15 +97,13 @@ pub use epi_backend::{run, NativeOptions};
 
 pub use egui_winit;
 
-use glium::glutin;
-
 // ----------------------------------------------------------------------------
 
-/// Use [`egui`] from a [`glium`] app.
+/// Convenience wrapper for using [`egui`] from a [`glium`] app.
 pub struct EguiGlium {
-    egui_ctx: egui::CtxRef,
-    egui_winit: egui_winit::State,
-    painter: crate::Painter,
+    pub egui_ctx: egui::CtxRef,
+    pub egui_winit: egui_winit::State,
+    pub painter: crate::Painter,
 }
 
 impl EguiGlium {
@@ -117,27 +115,6 @@ impl EguiGlium {
         }
     }
 
-    pub fn ctx(&self) -> &egui::CtxRef {
-        &self.egui_ctx
-    }
-
-    /// useful for calling e.g. [`crate::Painter::register_glium_texture`].
-    pub fn painter_mut(&mut self) -> &mut crate::Painter {
-        &mut self.painter
-    }
-
-    pub fn ctx_and_painter_mut(&mut self) -> (&egui::CtxRef, &mut crate::Painter) {
-        (&self.egui_ctx, &mut self.painter)
-    }
-
-    pub fn pixels_per_point(&self) -> f32 {
-        self.egui_winit.pixels_per_point()
-    }
-
-    pub fn egui_input(&self) -> &egui::RawInput {
-        self.egui_winit.egui_input()
-    }
-
     /// Returns `true` if egui wants exclusive use of this event
     /// (e.g. a mouse click on an egui window, or entering text into a text field).
     /// For instance, if you use egui for a game, you want to first call this
@@ -148,40 +125,20 @@ impl EguiGlium {
         self.egui_winit.on_event(&self.egui_ctx, event)
     }
 
-    /// Is this a close event or a Cmd-Q/Alt-F4 keyboard command?
-    pub fn is_quit_event(&self, event: &glutin::event::WindowEvent<'_>) -> bool {
-        self.egui_winit.is_quit_event(event)
-    }
-
-    pub fn begin_frame(&mut self, display: &glium::Display) {
-        let raw_input = self.take_raw_input(display);
-        self.begin_frame_with_input(raw_input);
-    }
-
-    pub fn begin_frame_with_input(&mut self, raw_input: egui::RawInput) {
-        self.egui_ctx.begin_frame(raw_input);
-    }
-
-    /// Prepare for a new frame. Normally you would call [`Self::begin_frame`] instead.
-    pub fn take_raw_input(&mut self, display: &glium::Display) -> egui::RawInput {
-        self.egui_winit
-            .take_egui_input(display.gl_window().window())
-    }
-
     /// Returns `needs_repaint` and shapes to draw.
-    pub fn end_frame(
+    pub fn run(
         &mut self,
         display: &glium::Display,
+        run_ui: impl FnMut(&egui::CtxRef),
     ) -> (bool, Vec<egui::epaint::ClippedShape>) {
-        let (egui_output, shapes) = self.egui_ctx.end_frame();
+        let raw_input = self
+            .egui_winit
+            .take_egui_input(display.gl_window().window());
+        let (egui_output, shapes) = self.egui_ctx.run(raw_input, run_ui);
         let needs_repaint = egui_output.needs_repaint;
-        self.handle_output(display, egui_output);
-        (needs_repaint, shapes)
-    }
-
-    pub fn handle_output(&mut self, display: &glium::Display, output: egui::Output) {
         self.egui_winit
-            .handle_output(display.gl_window().window(), &self.egui_ctx, output);
+            .handle_output(display.gl_window().window(), &self.egui_ctx, egui_output);
+        (needs_repaint, shapes)
     }
 
     pub fn paint<T: glium::Surface>(
@@ -196,7 +153,7 @@ impl EguiGlium {
             target,
             self.egui_ctx.pixels_per_point(),
             clipped_meshes,
-            &self.egui_ctx.texture(),
+            &self.egui_ctx.font_image(),
         );
     }
 }
